import { Component, OnInit, ElementRef, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { AllReceiveReportService } from '../services/all-receive-report.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-all-receive-report',
  templateUrl: './all-receive-report.component.html',
  styleUrls: ['./all-receive-report.component.css'],
})
export class AllReceiveReportComponent implements OnInit {
  @ViewChild('transactionTable') transactionTable: ElementRef;
  countries: any[] = [];
  selectedReportType: string = 'Detail';
  receivedReportData: any[] = [];
  agents: any[] = [];
  summaryReportData: any[] = [];
  StausWiseReportData: any[] = [];
  isLoading: boolean = false;
  form: FormGroup;
  selectedFileFormat: any = '1';

  constructor(
    private allReceiveReportService: AllReceiveReportService,
    private formBuilder: FormBuilder,
    private sanitizer: DomSanitizer
  ) {}

  ngOnInit(): void {
    this.createForm();
    this.loadReceivedCountries();
  }

  createForm(): void {
    const currentStartDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentEndDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    this.form = this.formBuilder.group({
      fromDate: [currentStartDate, Validators.required],
      toDate: [currentEndDate, Validators.required],
      reportType: ['Detail', Validators.required],
      receiveCountry: ['', Validators.required],
      receiveAgent: ['', Validators.required],
      fileFormat: [''],
    });
  }

  loadReceivedCountries(): void {
    this.allReceiveReportService.getReceiveCountryData().subscribe(
      (data: any[]) => {
        this.countries = data;
        if (this.countries.length > 0) {
          this.form.patchValue({ receiveCountry: this.countries[0].CountryID });
          this.onReceiveCountryChange();
        }
      },
      (error) => {
        console.error('Error fetching received countries:', error);
      }
    );
  }

  onReceiveCountryChange(): void {
    const selectedReceiveCountry = this.form.get('receiveCountry').value;
    this.allReceiveReportService
      .getReceiveAgentData(selectedReceiveCountry)
      .subscribe(
        (data: any[]) => {
          this.agents = data;
          if (this.agents.length > 0) {
            this.form.get('receiveAgent').setValue(this.agents[0].Agent_ID);
          }
        },
        (error) => {
          console.error('Error fetching receive agents:', error);
        }
      );
  }

  displayByDate(): void {}

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Agentwise_Transaction_Sent${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  onReportTypeChange(event: any): void {
    this.selectedReportType = event.target.value;
    console.log(this.selectedReportType);
  }

  callApiBasedOnReportType(): void {
    if (!this.form.get('fromDate').value || !this.form.get('toDate').value) {
      alert('Please select both From Date and To Date.');
      return;
    }

    if (
      this.form.get('reportType').value === '' ||
      this.form.get('receiveCountry').value === '' ||
      this.form.get('receiveAgent').value === ''
    ) {
      alert('Please select all dropdowns.');
      return;
    }

    const requestData = {
      ReportType: this.form.get('reportType').value,
      CountryID: this.form.get('receiveCountry').value,
      AgentID: this.form.get('receiveAgent').value,
      FromDate: this.form.get('fromDate').value,
      ToDate: this.form.get('toDate').value,
    };

    if (this.selectedReportType === 'Detail') {
      this.isLoading = true;

      this.allReceiveReportService
        .fillGridAllReceiveTrxnReportDetail(requestData)
        .subscribe(
          (response: any[]) => {
            this.receivedReportData = response;
            console.log(this.receivedReportData);
            this.isLoading = false;
          },
          (error) => {
            console.error('Error calling Detail report API:', error);
            this.isLoading = false;
          }
        );
    } else if (this.selectedReportType === 'Summary') {
      this.isLoading = true;

      this.allReceiveReportService
        .fillGridAllReceiveTrxnReportSummary(requestData)
        .subscribe(
          (response: any[]) => {
            this.summaryReportData = response;
            this.receivedReportData = [];
            this.isLoading = false;
          },
          (error) => {
            console.error('Error calling Summary report API:', error);
            this.isLoading = false;
          }
        );
    } else if (this.selectedReportType === 'StausWise') {
      this.isLoading = true;

      this.allReceiveReportService
        .fillGridStatusWiseSummaryAllReceiveTrxnReport(requestData)
        .subscribe(
          (response: any[]) => {
            this.StausWiseReportData = response;
            this.summaryReportData = [];
            this.receivedReportData = [];
            this.isLoading = false;
          },
          (error) => {
            console.error(
              'Error calling StatusWise Summary report API:',
              error
            );
            this.isLoading = false;
            // Handle error here
          }
        );
    }
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  sanitize(html: string): SafeHtml {
    return this.sanitizer.bypassSecurityTrustHtml(html);
  }
}
