import { Component, TemplateRef, ViewChild, OnInit } from '@angular/core';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ComplianceUploadDataService } from '../services/compliance-upload-data.service';
import { getCacheData } from '../utils/helper';
 
@Component({
  selector: 'app-compliance-upload-data',
  templateUrl: './compliance-upload-data.component.html',
  styleUrls: ['./compliance-upload-data.component.css'],
})
export class ComplianceUploadDataComponent implements OnInit {
  bsModalRef: BsModalRef;
  @ViewChild('fileUploadModalTemplate') fileUploadModalTemplate: TemplateRef<any>;
  @ViewChild('viewTransactionLog') viewTransactionLog: TemplateRef<any>;
  fileType: string = 'excel';
 
  users: string[] = [];
  selectedUser: string = '';
  complianceData: any[] = [];
  filteredData: any[] = [];
  currentPage: number = 1;
  pageSize: number = 15;
  totalPages: number = 0;
  selectedFile: File | null = null;
  isLoading: boolean = false;
  filteringData: boolean = false; // Loader for filtering data
 
  selectedOption: string = 'username'; // Default selection
  selectedInput: string = ''; // Input for username or date
  showNoDataAlert: boolean = false; // Flag for no data alert
 
  constructor(
    private modalService: BsModalService,
    private complianceService: ComplianceUploadDataService
  ) {}
 
  ngOnInit() {
    this.fetchData();
  }
 
  openFileUploadModal() {
    this.bsModalRef = this.modalService.show(this.fileUploadModalTemplate, {
      class: 'modal-lg',
    });
  }
 
  closeFileUploadModal() {
    this.selectedFile = null;
    this.bsModalRef.hide();
  }
 
  fetchData() {
    this.isLoading = true;
 
    this.complianceService.getComplianceData().subscribe(
      (data: any[]) => {
        this.isLoading = false;
        this.complianceData = data || []; // Ensure data is assigned, even if empty
        this.extractUsers();
        this.filterData(); // Initial data load, also filter by default selection
      },
      (error) => {
        console.error(error);
        this.isLoading = false;
      }
    );
  }
 
  extractUsers() {
    this.users = Array.from(
      new Set(this.complianceData.map((item) => item.PM_USER_NM))
    );
  }
 
  openTransactionLogModal() {
    this.bsModalRef = this.modalService.show(this.viewTransactionLog, {
      class: 'modal-lg',
    });
  }
 
  onPageChange(page: number) {
    this.currentPage = page;
  }
 
  getPaginatedData() {
    const startIndex = (this.currentPage - 1) * this.pageSize;
    const endIndex = startIndex + this.pageSize;
    return this.filteredData.slice(startIndex, endIndex);
  }
 
  onFileSelected(event: any) {
    this.selectedFile = event.target.files[0];
  }
 
  uploadFile() {
    const fileType = this.fileType;
    const userName =  getCacheData('username');
    const userId = getCacheData('G_UserID');
 
    if (this.selectedFile) {
      this.isLoading = true;
 
      this.complianceService.uploadFile(fileType, userName, userId, this.selectedFile).subscribe(
        (response) => {
          alert('File uploaded successfully:');
          this.selectedFile = null;
          this.isLoading = false;
          this.fetchData(); // Refresh data after upload
          this.bsModalRef.hide();
        },
        (error) => {
          console.error('Error uploading file:', error);
          alert('File uploaded Failed:');
          this.bsModalRef.hide();
          this.selectedFile = null;
       
          this.isLoading = false;
        }
      );
    }
  }
 
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
 
  onShowCompliance() {
    if (this.selectedOption === 'username') {
      if (!this.isValidUsername(this.selectedInput)) {
        this.showNoDataAlert = true;
        this.filteredData = []; // Clear filtered data on invalid input
        return;
      } else {
        this.showNoDataAlert = false;
      }
    } else if (this.selectedOption === 'date') {
      // No need to convert date format, use input directly
      if (!this.isValidDateFormat(this.selectedInput)) {
        alert('Invalid date format. Please use DD/MM/YYYY format.'); // Alert for invalid format
        this.showNoDataAlert = false; // Reset no data alert flag
        return;
      }
    }
    this.filterData();
  }
 
  filterData() {
    const input = this.selectedInput.trim().toLowerCase();
    this.filteringData = true;
 
    setTimeout(() => {
      if (this.selectedOption === 'username') {
        this.filteredData = this.complianceData.filter(
          (item) => item.PM_USER_NM.toLowerCase().includes(input)
        );
      } else if (this.selectedOption === 'date') {
        this.filteredData = this.complianceData.filter(
          (item) => item.DatePM_DT === input
        );
      }
 
      this.filteringData = false;
      this.calculateTotalPages();
      this.currentPage = 1; // Reset pagination after filtering
      this.showNoDataAlert = this.selectedOption === 'username' && this.filteredData.length === 0;
      if (this.selectedOption === 'date' && this.filteredData.length === 0) {
        alert('No data available for the selected date.'); // Alert when no data is found
      }
    }, 0); // Remove simulated delay for immediate response
  }
 
  isValidUsername(input: string): boolean {
    // Add your username validation logic here
    // For example, check if the username is non-empty and contains only alphanumeric characters
    const usernameRegex = /^[a-zA-Z0-9]+$/;
    return usernameRegex.test(input);
  }
 
  isValidDateFormat(input: string): boolean {
    const dateFormatRegex = /^\d{2}\/\d{2}\/\d{4}$/;
    return dateFormatRegex.test(input);
  }
 
  calculateTotalPages() {
    this.totalPages = Math.ceil(this.filteredData.length / this.pageSize);
  }
  getCurrentPageCount(): string {
    return `${this.currentPage} / ${this.totalPages}`;
  }
  removeFile() {
    this.selectedFile = null;
    // Reset the file input value
    const fileInput: HTMLInputElement = document.getElementById('fileInput') as HTMLInputElement;
    if (fileInput) {
      fileInput.value = '';
    }
  }
  refreshData() {
    this.fetchData(); // Call fetchData() to reload data
  }
  clearFileSelection() {
    this.selectedFile = null;
  }

  
 
}