import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { RebateReportService } from '../services/rebate-report.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-rebate-report',
  templateUrl: './rebate-report.component.html',
  styleUrls: ['./rebate-report.component.css'],
})
export class RebateReportComponent implements OnInit {
  form: FormGroup;
  agentList: any[] = [];
  paymentModeList: any[] = [];
  gridData: any[] = [];
  isLoading: boolean = false; // Loader state
  selectedFileFormat: string = '1';

  constructor(
    private fb: FormBuilder,
    private rebateReportService: RebateReportService
  ) {}

  ngOnInit(): void {
    const currentStartDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentEndDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    this.form = this.fb.group({
      startDate: [currentStartDate, Validators.required],
      endDate: [currentEndDate, Validators.required],
      agent: ['', Validators.required],
      paymentMode: ['', Validators.required],
    });

    this.getAgentList();
  }

  getAgentList(): void {
    this.rebateReportService.getAgentList().subscribe(
      (data: any[]) => {
        this.agentList = data;
        if (this.agentList.length > 0) {
          this.form.get('agent').setValue('0');
          this.getPaymentModeList(0);
        }
      },
      (error) => {
        console.error('Error fetching agent list:', error);
      }
    );
  }

  getPaymentModeList(agentId: any): void {
    this.rebateReportService.getPaymentModeList(agentId).subscribe(
      (data: any[]) => {
        this.paymentModeList = data;
        this.form.get('paymentMode').setValue('0');
        // if (this.paymentModeList.length > 0) {
        //   this.onSubmit();
        // }
      },
      (error) => {
        console.error('Error fetching payment mode list:', error);
      }
    );
  }

  onSubmit(): void {
    if (this.form.valid) {
      const { startDate, endDate, agent, paymentMode } = this.form.value;
      if (
        !this.form.get('startDate').value ||
        !this.form.get('endDate').value
      ) {
        alert('Please select both From and To dates.');
      } else {
        this.getFillGrid(startDate, endDate, agent, paymentMode);
      }
    } else {
      this.form.markAllAsTouched();
    }
  }

  getFillGrid(
    startDate: any,
    endDate: any,
    agent: any,
    paymentMode: any
  ): void {
    this.isLoading = true; // Start loader
    this.rebateReportService
      .getFillGrid(agent, paymentMode, startDate, endDate)
      .subscribe(
        (data: any[]) => {
          this.isLoading = false; // Stop loader
          this.gridData = data;
          if (this.gridData.length === 0) {
            alert('No data found.');
          }
        },
        (error) => {
          this.isLoading = false; // Stop loader on error
          console.error('Error fetching grid data:', error);
        
        }
      );
  }

  get startDate() {
    return this.form.get('startDate');
  }

  get endDate() {
    return this.form.get('endDate');
  }

  onAgentChange(agentId: any): void {
    this.getPaymentModeList(agentId);
  }

  onPaymentModeChange(paymentModeId: any): void {}

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  exportToExcel(): void {
    let ws: XLSX.WorkSheet;

    // Convert data to worksheet
    ws = XLSX.utils.json_to_sheet(this.gridData);

    // Create a workbook and add the worksheet
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    // Determine file extension and save to file
    const fileExtension = this.selectedFileFormat === '1' ? 'xlsx' : 'csv';
    const fileName = `Rebate_Report_${new Date().getTime()}.${fileExtension}`;

    // Save the workbook
    XLSX.writeFile(wb, fileName);
  }
}
